@@ -5,7 +5,7 @@ MQTT5
 
 # Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 # SPDX-License-Identifier: Apache-2.0.
-from typing import Any, Callable
+from typing import Any, Callable, Union
 import _awscrt
 from concurrent.futures import Future
 from enum import IntEnum
@@ -609,7 +609,7 @@ class UnsubackReasonCode(IntEnum):
     Returned when the client was not authorized to unsubscribe from that topic filter on the server.
     """
 
-    TOPIC_NAME_INVALID = 144
+    TOPIC_FILTER_INVALID = 143
     """
     Returned when the topic filter was correctly formed but is not allowed for the client on the server.
     """
@@ -955,7 +955,8 @@ class ConnackPacket:
 
     Args:
         session_present (bool): True if the client rejoined an existing session on the server, false otherwise.
-        reason_code (ConnectReasonCode): Indicates either success or the reason for failure for the connection attempt. session_expiry_interval_sec (int): A time interval, in seconds, that the server will persist this connection's MQTT session state for.  If present, this value overrides any session expiry specified in the preceding CONNECT packet.
+        reason_code (ConnectReasonCode): Indicates either success or the reason for failure for the connection attempt.
+        session_expiry_interval_sec (int): A time interval, in seconds, that the server will persist this connection's MQTT session state for.  If present, this value overrides any session expiry specified in the preceding CONNECT packet.
         receive_maximum (int): The maximum amount of in-flight QoS 1 or 2 messages that the server is willing to handle at once. If omitted or None, the limit is based on the valid MQTT packet id space (65535).
         maximum_qos (QoS): The maximum message delivery quality of service that the server will allow on this connection.
         retain_available (bool): Indicates whether the server supports retained messages.  If None, retained messages are supported.
@@ -1083,12 +1084,12 @@ class UnsubackPacket:
     Args:
         reason_string (str): Additional diagnostic information about the result of the UNSUBSCRIBE attempt.
         user_properties (Sequence[UserProperty]): List of MQTT5 user properties included with the packet.
-        reason_codes (Sequence[DisconnectReasonCode]): A list of reason codes indicating the result of unsubscribing from each individual topic filter entry in the associated UNSUBSCRIBE packet.
+        reason_codes (Sequence[UnsubackReasonCode]): A list of reason codes indicating the result of unsubscribing from each individual topic filter entry in the associated UNSUBSCRIBE packet.
 
     """
     reason_string: str = None
     user_properties: 'Sequence[UserProperty]' = None
-    reason_codes: 'Sequence[DisconnectReasonCode]' = None
+    reason_codes: 'Sequence[UnsubackReasonCode]' = None
 
 
 @dataclass
@@ -1104,8 +1105,9 @@ class PublishPacket:
         message_expiry_interval_sec (int): Sent publishes - indicates the maximum amount of time allowed to elapse for message delivery before the server should instead delete the message (relative to a recipient). Received publishes - indicates the remaining amount of time (from the server's perspective) before the message would have been deleted relative to the subscribing client. If left None, indicates no expiration timeout.
         topic_alias (int): An integer value that is used to identify the Topic instead of using the Topic Name.  On outbound publishes, this will only be used if the outbound topic aliasing behavior has been set to Manual.
         response_topic (str): Opaque topic string intended to assist with request/response implementations.  Not internally meaningful to MQTT5 or this client.
-        correlation_data (Any): Opaque binary data used to correlate between publish messages, as a potential method for request-response implementation.  Not internally meaningful to MQTT5.
-        subscription_identifiers (Sequence[int]): The subscription identifiers of all the subscriptions this message matched.
+        correlation_data (Optional[Union[bytes, str]]): Deprecated, use `correlation_data_bytes` instead.  Opaque binary data used to correlate between publish messages, as a potential method for request-response implementation.  Not internally meaningful to MQTT5.  For incoming publishes, this will be a utf8 string (if correlation data exists and it's convertible to utf-8) or None (either it didn't exist, or did but wasn't convertible)
+        correlation_data_bytes (Optional[Union[bytes, str]]): Opaque binary data used to correlate between publish messages, as a potential method for request-response implementation.  Not internally meaningful to MQTT5.  For outbound publishes, this field takes priority over `correlation_data`.  For incoming publishes, this will be binary data if correlation data is set, otherwise it will be None.
+        subscription_identifiers (Sequence[int]): The subscription identifiers of all the subscriptions this message matched.  This field is ignored on outbound publishes (setting it is a protocol error).
         content_type (str): Property specifying the content type of the payload.  Not internally meaningful to MQTT5.
         user_properties (Sequence[UserProperty]): List of MQTT5 user properties included with the packet.
     """
@@ -1117,7 +1119,9 @@ class PublishPacket:
     message_expiry_interval_sec: int = None
     topic_alias: int = None
     response_topic: str = None
-    correlation_data: Any = None   # Unicode objects are converted to C strings using 'utf-8' encoding
+    correlation_data_bytes: 'Optional[Union[bytes, str]]' = None  # binary data if correlation data exists on the packet
+    # Deprecated.  Incoming publishes: a string if correlation data exists on the packet and is convertible to utf-8
+    correlation_data: 'Optional[Union[bytes, str]]' = None
     subscription_identifiers: 'Sequence[int]' = None  # ignore attempts to set but provide in received packets
     content_type: str = None
     user_properties: 'Sequence[UserProperty]' = None
@@ -1233,10 +1237,10 @@ class OperationStatisticsData:
     """Dataclass containing some simple statistics about the current state of the client's queue of operations
 
     Args:
-        incomplete_operation_count (int): total number of operations submitted to the client that have not yet been completed.  Unacked operations are a subset of this.
-        incomplete_operation_size (int): total packet size of operations submitted to the client that have not yet been completed.  Unacked operations are a subset of this.
-        unacked_operation_count (int): total number of operations that have been sent to the server and are waiting for a corresponding ACK before they can be completed.
-        unacked_operation_size (int): total packet size of operations that have been sent to the server and are waiting for a corresponding ACK before they can be completed.
+        incomplete_operation_count (int): Total number of operations submitted to the client that have not yet been completed.  Unacked operations are a subset of this.
+        incomplete_operation_size (int): Total packet size of operations submitted to the client that have not yet been completed.  Unacked operations are a subset of this.
+        unacked_operation_count (int): Total number of operations that have been sent to the server and are waiting for a corresponding ACK before they can be completed.
+        unacked_operation_size (int): Total packet size of operations that have been sent to the server and are waiting for a corresponding ACK before they can be completed.
     """
     incomplete_operation_count: int = 0
     incomplete_operation_size: int = 0
@@ -1391,11 +1395,15 @@ class _ClientCore:
 
     def _ws_handshake_transform(self, http_request_binding, http_headers_binding, native_userdata):
         if self._ws_handshake_transform_cb is None:
-            _awscrt.mqtt5_ws_handshake_transform_complete(None, native_userdata)
+            _awscrt.mqtt5_ws_handshake_transform_complete(None, native_userdata, 0)
             return
 
         def _on_complete(f):
-            _awscrt.mqtt5_ws_handshake_transform_complete(f.exception(), native_userdata)
+            error_code = 0
+            hs_exception = f.exception()
+            if isinstance(hs_exception, exceptions.AwsCrtError):
+                error_code = hs_exception.code
+            _awscrt.mqtt5_ws_handshake_transform_complete(f.exception(), native_userdata, error_code)
 
         future = Future()
         future.add_done_callback(_on_complete)
@@ -1443,8 +1451,18 @@ class _ClientCore:
         if topic_alias_exists:
             publish_packet.topic_alias = topic_alias
         publish_packet.response_topic = response_topic
-        publish_packet.correlation_data = correlation_data
-        if publish_packet.subscription_identifiers is not None:
+
+        # hacky workaround to maintain behavioral backwards compatibility with deprecated parameter
+        if correlation_data is not None:
+            # `correlation_data_bytes` always has the correlation data, as binary data
+            publish_packet.correlation_data_bytes = correlation_data
+            try:
+                # `correlation_data` contains the correlation data as a utf-8 string, if it can be converted
+                publish_packet.correlation_data = correlation_data.decode("utf-8")
+            except Exception:
+                pass
+
+        if subscription_identifiers_tuples is not None:
             publish_packet.subscription_identifiers = [subscription_identifier
                                                        for (subscription_identifier) in subscription_identifiers_tuples]
         publish_packet.content_type = content_type
@@ -1764,7 +1782,7 @@ class Client(NativeResource):
                                                  will.message_expiry_interval_sec,
                                                  will.topic_alias,
                                                  will.response_topic,
-                                                 will.correlation_data,
+                                                 will.correlation_data_bytes or will.correlation_data,
                                                  will.content_type,
                                                  will.user_properties,
                                                  client_options.session_behavior,
@@ -1775,6 +1793,7 @@ class Client(NativeResource):
                                                  client_options.max_reconnect_delay_ms,
                                                  client_options.min_connected_time_to_reset_reconnect_delay_ms,
                                                  client_options.ping_timeout_ms,
+                                                 client_options.connack_timeout_ms,
                                                  client_options.ack_timeout_sec,
                                                  client_options.topic_aliasing_options,
                                                  websocket_is_none,
@@ -1861,7 +1880,7 @@ class Client(NativeResource):
                                      publish_packet.message_expiry_interval_sec,
                                      publish_packet.topic_alias,
                                      publish_packet.response_topic,
-                                     publish_packet.correlation_data,
+                                     publish_packet.correlation_data_bytes or publish_packet.correlation_data,
                                      publish_packet.content_type,
                                      publish_packet.user_properties,
                                      puback)
