@@ -226,21 +226,51 @@ struct aws_credentials_provider_imds_options {
 };
 
 /*
- * Configuration options for the provider that sources credentials from ECS container metadata
- *
- * ECS creds provider can be used to access creds via either
- * relative uri to a fixed endpoint http://169.254.170.2,
- * or via a full uri specified by environment variables:
+ * Configuration options for the provider that sources credentials from ECS container metadata.
+ * The ECS creds provider can be used to access creds via either a relative URI to a fixed endpoint
+ * (http://169.254.170.2) or via a full URI specified by environment variables (in order of priority):
  * AWS_CONTAINER_CREDENTIALS_RELATIVE_URI
  * AWS_CONTAINER_CREDENTIALS_FULL_URI
- * AWS_CONTAINER_AUTHORIZATION_TOKEN
- * If both relative uri and absolute uri are set, relative uri
- * has higher priority. Token is used in auth header but only for
- * absolute uri.
- * While above information is used in request only, endpoint info
- * is needed when creating ecs provider to initiate the connection
- * manager, more specifically, host and http scheme (tls or not)
- * from endpoint are needed.
+ *
+ *`AWS_CONTAINER_CREDENTIALS_FULL_URI` URL must satisfy one of the following:
+ *  1. The URL begins with `https`.
+ *  2. The resolved IP address is within the loopback CIDR (IPv4 127.0.0.0/8, IPv6 ::1/128), ECS container address
+ *     (169.254.170.2), or EKS Pod Identity address (169.254.170.23 or fd00:ec2::23).
+ *
+ * For the Authorization token, there are two ways (in order of priority):
+ * 1. AWS_CONTAINER_AUTHORIZATION_TOKEN_FILE (an env var which contains the absolute path to the token file. The file
+ * will be re-read for each call to get credentials.)
+ * 2. AWS_CONTAINER_AUTHORIZATION_TOKEN (an env var that contains a static auth token)
+ */
+struct aws_credentials_provider_ecs_environment_options {
+    struct aws_credentials_provider_shutdown_options shutdown_options;
+
+    /*
+     * (Required)
+     * Connection bootstrap to use for any network connections made while sourcing credentials
+     */
+    struct aws_client_bootstrap *bootstrap;
+
+    /*
+     * (Required)
+     * Client TLS context to use when making a query. This will only be used if the AWS_CONTAINER_CREDENTIALS_FULL_URI
+     * is set and starts with https
+     */
+    struct aws_tls_ctx *tls_ctx;
+
+    /* For mocking the http layer in tests, leave NULL otherwise */
+    struct aws_auth_http_system_vtable *function_table;
+};
+
+/*
+ * Configuration options for the provider that sources credentials from ECS container metadata.
+ * This options struct doesn't read anything from the environment and requires everything to be explicitly passed in. If
+ * you need to read properties from the environment, use the `aws_credentials_provider_ecs_environment_options`.
+ *
+ *`host` must satisfy one of the following:
+ *  1. tls_context is set
+ *  2. The resolved IP address is within the loopback CIDR (IPv4 127.0.0.0/8, IPv6 ::1/128), ECS container address
+ *     (169.254.170.2), or EKS Pod Identity address (169.254.170.23 or fd00:ec2::23).
  */
 struct aws_credentials_provider_ecs_options {
     struct aws_credentials_provider_shutdown_options shutdown_options;
@@ -261,7 +291,15 @@ struct aws_credentials_provider_ecs_options {
     struct aws_byte_cursor path_and_query;
 
     /*
-     * Authorization token to include in the credentials query
+     * Authorization token file path to include in the credentials query. The file will be re-read for each call to
+     * get_credentials.
+     * This has higher priority than `auth_token`.
+     */
+    struct aws_byte_cursor auth_token_file_path;
+
+    /*
+     * Authorization token to include in the credentials query.
+     * No effect if `auth_token_file_path` is set.
      */
     struct aws_byte_cursor auth_token;
 
@@ -332,14 +370,14 @@ struct aws_credentials_provider_x509_options {
  ---------------------------------------------------------------------------------
  | Parameter           | Environment Variable Name    | Config File Property Name |
  ----------------------------------------------------------------------------------
- | region              | AWS_DEFAULT_REGION           | region                    |
+ | region              | AWS_REGION/AWS_DEFAULT_REGION| region                    |
  | role_arn            | AWS_ROLE_ARN                 | role_arn                  |
  | role_session_name   | AWS_ROLE_SESSION_NAME        | role_session_name         |
  | token_file_path     | AWS_WEB_IDENTITY_TOKEN_FILE  | web_identity_token_file   |
  |--------------------------------------------------------------------------------|
  * The order of resolution is the following
  * 1. Parameters
- * 2. Environment Variables
+ * 2. Environment Variables (in case of region, the AWS_REGION is preferred over the AWS_DEFAULT_REGION)
  * 3. Config File
  */
 struct aws_credentials_provider_sts_web_identity_options {
@@ -437,7 +475,12 @@ struct aws_credentials_provider_sso_options {
 };
 
 /**
- * Configuration options for the STS credentials provider
+ * Configuration options for the STS credentials provider.
+ * STS Credentials Provider will try to automatically resolve the region and use a regional STS endpoint if successful.
+ * The region resolution order is the following:
+ * 1. AWS_REGION environment variable
+ * 2. AWS_DEFAULT_REGION environment variable
+ * 3. The region property in the config file.
  */
 struct aws_credentials_provider_sts_options {
     /*
@@ -466,6 +509,11 @@ struct aws_credentials_provider_sts_options {
      */
     struct aws_byte_cursor session_name;
 
+    /*
+     * (Optional) Unique identifier for assuming a role in another account
+     */
+    struct aws_byte_cursor external_id;
+
     /*
      * How long sourced credentials should remain valid for, in seconds.  900 is the minimum allowed value.
      */
@@ -476,6 +524,26 @@ struct aws_credentials_provider_sts_options {
      */
     const struct aws_http_proxy_options *http_proxy_options;
 
+    /**
+     * (Optional)
+     * Uses a cached config file profile collection (~/.aws/config). You can also pass a merged profile collection,
+     * which contains both a config file and a credentials file.
+     * If provided, config_file_name_override is ignored.
+     */
+    struct aws_profile_collection *profile_collection_cached;
+
+    /*
+     * (Optional)
+     * Override of what profile to use; if not set, 'default' will be used.
+     */
+    struct aws_byte_cursor profile_name_override;
+
+    /*
+     * (Optional)
+     * Override path to the profile config file (~/.aws/config by default).
+     */
+    struct aws_byte_cursor config_file_name_override;
+
     struct aws_credentials_provider_shutdown_options shutdown_options;
 
     /* For mocking, leave NULL otherwise */
@@ -545,7 +613,6 @@ struct aws_credentials_provider_chain_default_options {
      * Use a cached merged profile collection. A merge collection has both config file
      * (~/.aws/config) and credentials file based profile collection (~/.aws/credentials) using
      * `aws_profile_collection_new_from_merge`.
-     * If this option is provided, `config_file_name_override` and `credentials_file_name_override` will be ignored.
      */
     struct aws_profile_collection *profile_collection_cached;
 
@@ -816,7 +883,7 @@ bool aws_credentials_is_anonymous(const struct aws_credentials *credentials);
  * the hybrid mode based on AWS credentials.
  *
  * @param allocator memory allocator to use for all memory allocation
- * @param credentials AWS credentials to derive the ECC key from using the AWS sigv4a key deriviation specification
+ * @param credentials AWS credentials to derive the ECC key from using the AWS sigv4a key derivation specification
  * @return a new ecc key pair or NULL on failure
  */
 AWS_AUTH_API
@@ -986,8 +1053,24 @@ struct aws_credentials_provider *aws_credentials_provider_new_imds(
     struct aws_allocator *allocator,
     const struct aws_credentials_provider_imds_options *options);
 
+/**
+ * Creates a provider that sources credentials from the ecs role credentials service and reads the required params from
+ * environment variables
+ *
+ * @param allocator memory allocator to use for all memory allocation
+ * @param options provider-specific configuration options
+ *
+ * @return the newly-constructed credentials provider, or NULL if an error occurred.
+ */
+AWS_AUTH_API
+struct aws_credentials_provider *aws_credentials_provider_new_ecs_from_environment(
+    struct aws_allocator *allocator,
+    const struct aws_credentials_provider_ecs_environment_options *options);
+
 /**
  * Creates a provider that sources credentials from the ecs role credentials service
+ * This function doesn't read anything from the environment and requires everything to be explicitly passed in.
+ * If you need to read properties from the environment, use the `aws_credentials_provider_new_ecs_from_environment`.
  *
  * @param allocator memory allocator to use for all memory allocation
  * @param options provider-specific configuration options
