@@ -82,6 +82,7 @@ static struct aws_byte_cursor s_sec_websocket_key_header_name;
 static struct aws_byte_cursor s_sec_websocket_protocol_header_name;
 static struct aws_byte_cursor s_sec_websocket_version_header_name;
 static struct aws_byte_cursor s_upgrade_header_name;
+static struct aws_byte_cursor s_transfer_encoding_header_name;
 
 static struct aws_byte_cursor s_amz_content_sha256_header_name;
 static struct aws_byte_cursor s_amz_date_header_name;
@@ -157,6 +158,11 @@ int aws_signing_init_signing_tables(struct aws_allocator *allocator) {
         return AWS_OP_ERR;
     }
 
+    s_transfer_encoding_header_name = aws_byte_cursor_from_c_str("transfer-encoding");
+    if (aws_hash_table_put(&s_skipped_headers, &s_transfer_encoding_header_name, NULL, NULL)) {
+        return AWS_OP_ERR;
+    }
+
     if (aws_hash_table_init(
             &s_forbidden_headers,
             allocator,
@@ -1351,6 +1357,20 @@ static int s_build_canonical_stable_header_list(
         }
 
         /* NOTE: Update MAX_AUTHORIZATION_HEADER_COUNT if more headers added */
+    } else if (
+        state->config.signature_type == AWS_ST_HTTP_REQUEST_QUERY_PARAMS &&
+        aws_byte_cursor_eq_c_str(&state->config.service, "vpc-lattice-svcs")) {
+        /* NOTES: TEMPORAY WORKAROUND FOR VPC Lattice. SHALL BE REMOVED IN NEAR FUTURE */
+        /* Add unsigned payload as `x-amz-content-sha256` header to the canonical request when signing through query
+         * params.  */
+        if (s_add_authorization_header(
+                state,
+                stable_header_list,
+                out_required_capacity,
+                s_amz_content_sha256_header_name,
+                g_aws_signed_body_value_unsigned_payload)) {
+            return AWS_OP_ERR;
+        }
     }
 
     *out_required_capacity += aws_array_list_length(stable_header_list) * 2; /*  ':' + '\n' per header */
@@ -1513,6 +1533,17 @@ static int s_build_canonical_payload(struct aws_signing_state_aws *state) {
     struct aws_hash *hash = NULL;
 
     int result = AWS_OP_ERR;
+    if (state->config.signature_type == AWS_ST_HTTP_REQUEST_QUERY_PARAMS &&
+        aws_byte_cursor_eq_c_str(&state->config.service, "vpc-lattice-svcs")) {
+        /* NOTES: TEMPORAY WORKAROUND FOR VPC Lattice. SHALL BE REMOVED IN NEAR FUTURE */
+        /* ALWAYS USE UNSIGNED-PAYLOAD FOR VPC Lattice.  */
+        if (aws_byte_buf_append_dynamic(payload_hash_buffer, &g_aws_signed_body_value_unsigned_payload) ==
+            AWS_OP_SUCCESS) {
+            result = AWS_OP_SUCCESS;
+        }
+        goto on_cleanup;
+    }
+
     if (state->config.signed_body_value.len == 0) {
         /* No value provided by user, so we must calculate it */
         hash = aws_sha256_new(allocator);
