@@ -18,7 +18,7 @@
 #include <aws/io/tls_channel_handler.h>
 #include <aws/io/uri.h>
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #    pragma warning(disable : 4204) /* non-constant aggregate initializer */
 #    pragma warning(disable : 4232) /* function pointer to dll symbol */
 #endif
@@ -39,7 +39,7 @@ AWS_STATIC_STRING_FROM_LITERAL(s_proxy_no_verify_peer_env_var, "AWS_PROXY_NO_VER
 #endif
 
 static struct aws_http_proxy_system_vtable s_default_vtable = {
-    .setup_client_tls = &aws_channel_setup_client_tls,
+    .aws_channel_setup_client_tls = &aws_channel_setup_client_tls,
 };
 
 static struct aws_http_proxy_system_vtable *s_vtable = &s_default_vtable;
@@ -162,6 +162,7 @@ struct aws_http_proxy_user_data *aws_http_proxy_user_data_new(
     user_data->original_channel_on_setup = on_channel_setup;
     user_data->original_channel_on_shutdown = on_channel_shutdown;
     user_data->requested_event_loop = options.requested_event_loop;
+    user_data->host_resolution_config = options.host_resolution_config;
     user_data->prior_knowledge_http2 = options.prior_knowledge_http2;
 
     /* one and only one setup callback must be valid */
@@ -465,7 +466,7 @@ static struct aws_http_message *s_build_h1_proxy_connect_request(struct aws_http
     }
 
     char port_str[20] = "\0";
-    snprintf(port_str, sizeof(port_str), "%d", (int)user_data->original_port);
+    snprintf(port_str, sizeof(port_str), "%u", user_data->original_port);
     struct aws_byte_cursor port_cursor = aws_byte_cursor_from_c_str(port_str);
     if (aws_byte_buf_append(&path_buffer, &port_cursor)) {
         goto on_error;
@@ -771,7 +772,7 @@ static void s_aws_http_on_stream_complete_tunnel_proxy(
             last_slot = last_slot->adj_right;
         }
 
-        if (s_vtable->setup_client_tls(last_slot, context->original_tls_options)) {
+        if (s_vtable->aws_channel_setup_client_tls(last_slot, context->original_tls_options)) {
             AWS_LOGF_ERROR(
                 AWS_LS_HTTP_CONNECTION,
                 "(%p) Proxy connection failed to start TLS negotiation with error %d(%s)",
@@ -1048,6 +1049,7 @@ static int s_aws_http_client_connect_via_forwarding_proxy(const struct aws_http_
     options_copy.on_shutdown = s_aws_http_on_client_connection_http_proxy_shutdown_fn;
     options_copy.tls_options = options->proxy_options->tls_options;
     options_copy.requested_event_loop = options->requested_event_loop;
+    options_copy.host_resolution_config = options->host_resolution_config;
     options_copy.prior_knowledge_http2 = false; /* ToDo, expose the protocol specific config for proxy connection. */
 
     int result = aws_http_client_connect_internal(&options_copy, s_proxy_http_request_transform);
@@ -1084,6 +1086,7 @@ static int s_create_tunneling_connection(struct aws_http_proxy_user_data *user_d
     connect_options.http1_options = NULL; /* ToDo, expose the protocol specific config for proxy connection. */
     connect_options.http2_options = NULL; /* ToDo */
     connect_options.requested_event_loop = user_data->requested_event_loop;
+    connect_options.host_resolution_config = user_data->host_resolution_config;
 
     int result = aws_http_client_connect(&connect_options);
     if (result == AWS_OP_ERR) {
@@ -1125,18 +1128,35 @@ static int s_aws_http_client_connect_via_tunneling_proxy(
 
 static enum aws_http_proxy_connection_type s_determine_proxy_connection_type(
     enum aws_http_proxy_connection_type proxy_connection_type,
-    const struct aws_tls_connection_options *tls_options) {
+    bool is_tls_connection) {
     if (proxy_connection_type != AWS_HPCT_HTTP_LEGACY) {
         return proxy_connection_type;
     }
 
-    if (tls_options != NULL) {
+    if (is_tls_connection) {
         return AWS_HPCT_HTTP_TUNNEL;
     } else {
         return AWS_HPCT_HTTP_FORWARD;
     }
 }
 
+static struct aws_string *s_get_proxy_environment_value(
+    struct aws_allocator *allocator,
+    const struct aws_string *env_name) {
+    struct aws_string *out_string = NULL;
+    if (aws_get_environment_value(allocator, env_name, &out_string) == AWS_OP_SUCCESS && out_string != NULL &&
+        out_string->len > 0) {
+        AWS_LOGF_DEBUG(
+            AWS_LS_HTTP_CONNECTION,
+            "%s environment found, %s",
+            aws_string_c_str(env_name),
+            aws_string_c_str(out_string));
+        return out_string;
+    }
+    aws_string_destroy(out_string);
+    return NULL;
+}
+
 static int s_proxy_uri_init_from_env_variable(
     struct aws_allocator *allocator,
     const struct aws_http_client_connection_options *options,
@@ -1145,25 +1165,19 @@ static int s_proxy_uri_init_from_env_variable(
     struct aws_string *proxy_uri_string = NULL;
     *found = false;
     if (options->tls_options) {
-        if (aws_get_environment_value(allocator, s_https_proxy_env_var_low, &proxy_uri_string) == AWS_OP_SUCCESS &&
-            proxy_uri_string != NULL) {
-            AWS_LOGF_DEBUG(AWS_LS_HTTP_CONNECTION, "https_proxy environment found");
-        } else if (
-            aws_get_environment_value(allocator, s_https_proxy_env_var, &proxy_uri_string) == AWS_OP_SUCCESS &&
-            proxy_uri_string != NULL) {
-            AWS_LOGF_DEBUG(AWS_LS_HTTP_CONNECTION, "HTTPS_PROXY environment found");
-        } else {
+        proxy_uri_string = s_get_proxy_environment_value(allocator, s_https_proxy_env_var_low);
+        if (proxy_uri_string == NULL) {
+            proxy_uri_string = s_get_proxy_environment_value(allocator, s_https_proxy_env_var);
+        }
+        if (proxy_uri_string == NULL) {
             return AWS_OP_SUCCESS;
         }
     } else {
-        if (aws_get_environment_value(allocator, s_http_proxy_env_var_low, &proxy_uri_string) == AWS_OP_SUCCESS &&
-            proxy_uri_string != NULL) {
-            AWS_LOGF_DEBUG(AWS_LS_HTTP_CONNECTION, "http_proxy environment found");
-        } else if (
-            aws_get_environment_value(allocator, s_http_proxy_env_var, &proxy_uri_string) == AWS_OP_SUCCESS &&
-            proxy_uri_string != NULL) {
-            AWS_LOGF_DEBUG(AWS_LS_HTTP_CONNECTION, "HTTP_PROXY environment found");
-        } else {
+        proxy_uri_string = s_get_proxy_environment_value(allocator, s_http_proxy_env_var_low);
+        if (proxy_uri_string == NULL) {
+            proxy_uri_string = s_get_proxy_environment_value(allocator, s_http_proxy_env_var);
+        }
+        if (proxy_uri_string == NULL) {
             return AWS_OP_SUCCESS;
         }
     }
@@ -1184,7 +1198,7 @@ static int s_connect_proxy(const struct aws_http_client_connection_options *opti
     }
 
     enum aws_http_proxy_connection_type proxy_connection_type =
-        s_determine_proxy_connection_type(options->proxy_options->connection_type, options->tls_options);
+        s_determine_proxy_connection_type(options->proxy_options->connection_type, options->tls_options != NULL);
 
     switch (proxy_connection_type) {
         case AWS_HPCT_HTTP_FORWARD:
@@ -1213,7 +1227,7 @@ static int s_setup_proxy_tls_env_variable(
             AWS_LS_HTTP_CONNECTION,
             "Failed making default TLS context because of BYO_CRYPTO, set up the tls_options for proxy_env_settings to "
             "make it work.");
-        return AWS_OP_ERR;
+        return aws_raise_error(AWS_ERROR_UNIMPLEMENTED);
 #else
         struct aws_tls_ctx *tls_ctx = NULL;
         struct aws_tls_ctx_options tls_ctx_options;
@@ -1332,10 +1346,10 @@ static struct aws_http_proxy_config *s_aws_http_proxy_config_new(
         return NULL;
     }
 
+    config->allocator = allocator;
     config->connection_type = override_proxy_connection_type;
 
     if (aws_byte_buf_init_copy_from_cursor(&config->host, allocator, proxy_options->host)) {
-
         goto on_error;
     }
 
@@ -1346,7 +1360,6 @@ static struct aws_http_proxy_config *s_aws_http_proxy_config_new(
         }
     }
 
-    config->allocator = allocator;
     config->port = proxy_options->port;
 
     if (proxy_options->proxy_strategy != NULL) {
@@ -1399,7 +1412,7 @@ struct aws_http_proxy_config *aws_http_proxy_config_new_from_connection_options(
     return s_aws_http_proxy_config_new(
         allocator,
         options->proxy_options,
-        s_determine_proxy_connection_type(options->proxy_options->connection_type, options->tls_options));
+        s_determine_proxy_connection_type(options->proxy_options->connection_type, options->tls_options != NULL));
 }
 
 struct aws_http_proxy_config *aws_http_proxy_config_new_from_manager_options(
@@ -1411,7 +1424,8 @@ struct aws_http_proxy_config *aws_http_proxy_config_new_from_manager_options(
     return s_aws_http_proxy_config_new(
         allocator,
         options->proxy_options,
-        s_determine_proxy_connection_type(options->proxy_options->connection_type, options->tls_connection_options));
+        s_determine_proxy_connection_type(
+            options->proxy_options->connection_type, options->tls_connection_options != NULL));
 }
 
 struct aws_http_proxy_config *aws_http_proxy_config_new_tunneling_from_proxy_options(
@@ -1432,6 +1446,16 @@ struct aws_http_proxy_config *aws_http_proxy_config_new_from_proxy_options(
     return s_aws_http_proxy_config_new(allocator, proxy_options, proxy_options->connection_type);
 }
 
+struct aws_http_proxy_config *aws_http_proxy_config_new_from_proxy_options_with_tls_info(
+    struct aws_allocator *allocator,
+    const struct aws_http_proxy_options *proxy_options,
+    bool is_tls_connection) {
+    AWS_FATAL_ASSERT(proxy_options != NULL);
+
+    return s_aws_http_proxy_config_new(
+        allocator, proxy_options, s_determine_proxy_connection_type(proxy_options->connection_type, is_tls_connection));
+}
+
 struct aws_http_proxy_config *aws_http_proxy_config_new_clone(
     struct aws_allocator *allocator,
     const struct aws_http_proxy_config *proxy_config) {
@@ -1632,6 +1656,7 @@ int aws_http_proxy_new_socket_channel(
     http_connection_options.on_setup = NULL;    /* use channel callbacks, not http callbacks */
     http_connection_options.on_shutdown = NULL; /* use channel callbacks, not http callbacks */
     http_connection_options.requested_event_loop = channel_options->requested_event_loop;
+    http_connection_options.host_resolution_config = channel_options->host_resolution_override_config;
 
     if (s_aws_http_client_connect_via_tunneling_proxy(
             &http_connection_options, s_http_proxied_socket_channel_setup, s_http_proxied_socket_channel_shutdown)) {
