@@ -10,6 +10,7 @@
 #include <aws/common/byte_buf.h>
 #include <aws/common/encoding.h>
 #include <aws/common/string.h>
+#include <aws/common/uri.h>
 #include <aws/http/request_response.h>
 #include <aws/io/async_stream.h>
 #include <aws/io/stream.h>
@@ -20,6 +21,12 @@ const struct aws_byte_cursor g_s3_create_multipart_upload_excluded_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-MD5"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc64nvme"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc32c"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc32"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-sha1"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-sha256"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("if-none-match"),
 };
 
 const size_t g_s3_create_multipart_upload_excluded_headers_count =
@@ -49,6 +56,12 @@ const struct aws_byte_cursor g_s3_upload_part_excluded_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-mode"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-retain-until-date"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-legal-hold"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc64nvme"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc32c"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc32"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-sha1"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-sha256"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("if-none-match"),
 };
 
 const size_t g_s3_upload_part_excluded_headers_count = AWS_ARRAY_SIZE(g_s3_upload_part_excluded_headers);
@@ -82,6 +95,7 @@ const struct aws_byte_cursor g_s3_complete_multipart_upload_excluded_headers[] =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-legal-hold"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-mp-object-size"),
 };
 
 const size_t g_s3_complete_multipart_upload_excluded_headers_count =
@@ -116,6 +130,7 @@ const struct aws_byte_cursor g_s3_complete_multipart_upload_with_checksum_exclud
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-sdk-checksum-algorithm"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-mp-object-size"),
 };
 
 const struct aws_byte_cursor g_s3_list_parts_excluded_headers[] = {
@@ -211,10 +226,15 @@ const struct aws_byte_cursor g_s3_abort_multipart_upload_excluded_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-legal-hold"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("if-none-match"),
 };
 
 static const struct aws_byte_cursor s_x_amz_meta_prefix = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-meta-");
 
+static const struct aws_byte_cursor s_checksum_type_header =
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-type");
+static const struct aws_byte_cursor s_checksum_type_full_object = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("full_object");
+
 const size_t g_s3_abort_multipart_upload_excluded_headers_count =
     AWS_ARRAY_SIZE(g_s3_abort_multipart_upload_excluded_headers);
 
@@ -245,11 +265,11 @@ struct aws_http_message *aws_s3_ranged_get_object_message_new(
     return message;
 }
 
-/* Creates a create-multipart-upload request from a given put objet request. */
+/* Creates a create-multipart-upload request from a given put object request. */
 struct aws_http_message *aws_s3_create_multipart_upload_message_new(
     struct aws_allocator *allocator,
     struct aws_http_message *base_message,
-    enum aws_s3_checksum_algorithm algorithm) {
+    const struct checksum_config_storage *checksum_config) {
     AWS_PRECONDITION(allocator);
 
     /* For multipart upload, some headers should ONLY be in the initial create-multipart request.
@@ -282,13 +302,27 @@ struct aws_http_message *aws_s3_create_multipart_upload_message_new(
             goto error_clean_up;
         }
     }
-    if (algorithm) {
-        if (aws_http_headers_set(
-                headers,
-                g_create_mpu_checksum_header_name,
-                *aws_get_create_mpu_header_name_from_algorithm(algorithm))) {
-            goto error_clean_up;
+
+    if (checksum_config && (checksum_config->location != AWS_SCL_NONE || checksum_config->has_full_object_checksum)) {
+        if (checksum_config->checksum_algorithm) {
+            if (aws_http_headers_set(
+                    headers,
+                    g_checksum_algorithm_header_name,
+                    aws_get_checksum_algorithm_name(checksum_config->checksum_algorithm))) {
+                goto error_clean_up;
+            }
         }
+        if (checksum_config->has_full_object_checksum) {
+            /* Request S3 to store the full object checksum as it's set from user. */
+            if (aws_http_headers_set(headers, s_checksum_type_header, s_checksum_type_full_object)) {
+                goto error_clean_up;
+            }
+        }
+    }
+
+    struct aws_byte_cursor content_length_cursor = aws_byte_cursor_from_c_str("0");
+    if (aws_http_headers_set(headers, g_content_length_header_name, content_length_cursor)) {
+        goto error_clean_up;
     }
 
     aws_http_message_set_request_method(message, g_post_method);
@@ -310,7 +344,7 @@ struct aws_http_message *aws_s3_upload_part_message_new(
     uint32_t part_number,
     const struct aws_string *upload_id,
     bool should_compute_content_md5,
-    const struct checksum_config *checksum_config,
+    const struct checksum_config_storage *checksum_config,
     struct aws_byte_buf *encoded_checksum_output) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(base_message);
@@ -424,30 +458,63 @@ static const struct aws_byte_cursor s_slash_char = AWS_BYTE_CUR_INIT_FROM_STRING
  */
 struct aws_http_message *aws_s3_get_source_object_size_message_new(
     struct aws_allocator *allocator,
-    struct aws_http_message *base_message) {
-    struct aws_http_message *message = NULL;
+    struct aws_http_message *base_message,
+    struct aws_uri *source_uri) {
+
+    struct aws_http_message *message = aws_http_message_new_request(allocator);
     struct aws_byte_buf head_object_host_header;
     AWS_ZERO_STRUCT(head_object_host_header);
 
+    if (message == NULL) {
+        goto error_cleanup;
+    }
+
+    if (aws_http_message_set_request_method(message, g_head_method)) {
+        goto error_cleanup;
+    }
+    if (source_uri != NULL && source_uri->self_size > 0) {
+        /* Parse source host header and path from the provided URI */
+        struct aws_byte_cursor host = *aws_uri_host_name(source_uri);
+        struct aws_byte_cursor path = *aws_uri_path(source_uri);
+        if (host.len == 0 || path.len == 0) {
+            aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+            goto error_cleanup;
+        }
+        struct aws_http_header host_header = {
+            .name = g_host_header_name,
+            .value = host,
+        };
+        if (aws_http_message_add_header(message, host_header)) {
+            goto error_cleanup;
+        }
+
+        if (aws_http_message_set_request_path(message, path)) {
+            goto error_cleanup;
+        }
+        return message;
+    }
+
+    /* Parse the source host header and path from the x-amz-copy-source header and the destination URI */
+
     AWS_PRECONDITION(allocator);
 
     /* Find the x-amz-copy-source header, to extract source bucket/key information. */
     struct aws_http_headers *headers = aws_http_message_get_headers(base_message);
     if (!headers) {
         AWS_LOGF_ERROR(AWS_LS_S3_GENERAL, "CopyRequest is missing headers");
-        return NULL;
+        goto error_cleanup;
     }
 
     struct aws_byte_cursor source_header;
     const struct aws_byte_cursor copy_source_header = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source");
     if (aws_http_headers_get(headers, copy_source_header, &source_header) != AWS_OP_SUCCESS) {
         AWS_LOGF_ERROR(AWS_LS_S3_GENERAL, "CopyRequest is missing the x-amz-copy-source header");
-        return NULL;
+        goto error_cleanup;
     }
     struct aws_byte_cursor host;
     if (aws_http_headers_get(headers, g_host_header_name, &host) != AWS_OP_SUCCESS) {
         AWS_LOGF_ERROR(AWS_LS_S3_GENERAL, "CopyRequest is missing the Host header");
-        return NULL;
+        goto error_cleanup;
     }
 
     struct aws_byte_cursor request_path = source_header;
@@ -496,15 +563,6 @@ struct aws_http_message *aws_s3_get_source_object_size_message_new(
         goto error_cleanup;
     }
 
-    message = aws_http_message_new_request(allocator);
-    if (message == NULL) {
-        goto error_cleanup;
-    }
-
-    if (aws_http_message_set_request_method(message, g_head_method)) {
-        goto error_cleanup;
-    }
-
     struct aws_http_header host_header = {
         .name = g_host_header_name,
         .value = aws_byte_cursor_from_buf(&head_object_host_header),
@@ -553,32 +611,37 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
     struct aws_byte_buf *body_buffer,
     const struct aws_string *upload_id,
     const struct aws_array_list *parts,
-    enum aws_s3_checksum_algorithm algorithm) {
+    const struct checksum_config_storage *checksum_config) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(base_message);
     AWS_PRECONDITION(body_buffer);
     AWS_PRECONDITION(upload_id);
     AWS_PRECONDITION(parts);
 
-    const struct aws_byte_cursor *mpu_algorithm_checksum_name = aws_get_complete_mpu_name_from_algorithm(algorithm);
-
+    struct aws_byte_cursor mpu_algorithm_checksum_name;
+    AWS_ZERO_STRUCT(mpu_algorithm_checksum_name);
     struct aws_http_message *message = NULL;
-    if (algorithm == AWS_SCA_NONE) {
-        /* We don't need to worry about the pre-calculated checksum from user as for multipart upload, only way to
-         * calculate checksum is from client. */
-        message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
-            allocator,
-            base_message,
-            g_s3_complete_multipart_upload_excluded_headers,
-            AWS_ARRAY_SIZE(g_s3_complete_multipart_upload_excluded_headers),
-            true /*exclude_x_amz_meta*/);
-    } else {
+    bool set_checksums =
+        checksum_config && (checksum_config->location != AWS_SCL_NONE || checksum_config->has_full_object_checksum);
+    const struct aws_http_headers *initial_message_headers = aws_http_message_get_headers(base_message);
+    AWS_ASSERT(initial_message_headers);
+    if (set_checksums) {
+        mpu_algorithm_checksum_name =
+            aws_get_completed_part_name_from_checksum_algorithm(checksum_config->checksum_algorithm);
         message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
             allocator,
             base_message,
             g_s3_complete_multipart_upload_with_checksum_excluded_headers,
             AWS_ARRAY_SIZE(g_s3_complete_multipart_upload_with_checksum_excluded_headers),
             true /*exclude_x_amz_meta*/);
+
+    } else {
+        message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
+            allocator,
+            base_message,
+            g_s3_complete_multipart_upload_excluded_headers,
+            AWS_ARRAY_SIZE(g_s3_complete_multipart_upload_excluded_headers),
+            true /*exclude_x_amz_meta*/);
     }
 
     struct aws_http_headers *headers = NULL;
@@ -598,6 +661,28 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
     if (headers == NULL) {
         goto error_clean_up;
     }
+    if (set_checksums && checksum_config->has_full_object_checksum) {
+        /* Set the full object checksum header. */
+        AWS_ASSERT(checksum_config->checksum_algorithm != AWS_SCA_NONE);
+        if (aws_http_headers_set(
+                headers,
+                aws_get_http_header_name_from_checksum_algorithm(checksum_config->checksum_algorithm),
+                aws_byte_cursor_from_buf(&checksum_config->full_object_checksum))) {
+            goto error_clean_up;
+        }
+        /* Request S3 to store the full object checksum as it's set from user. */
+        if (aws_http_headers_set(headers, s_checksum_type_header, s_checksum_type_full_object)) {
+            goto error_clean_up;
+        }
+    }
+    struct aws_byte_cursor content_length_cursor;
+    if (aws_http_headers_get(initial_message_headers, g_content_length_header_name, &content_length_cursor) ==
+        AWS_OP_SUCCESS) {
+        /* Set content-length from base message as x-amz-mp-object-size. */
+        if (aws_http_headers_set(headers, aws_byte_cursor_from_c_str("x-amz-mp-object-size"), content_length_cursor)) {
+            goto error_clean_up;
+        }
+    }
 
     /* Create XML payload with all the etags of finished parts */
     {
@@ -641,13 +726,13 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
                 goto error_clean_up;
             }
 
-            if (mpu_algorithm_checksum_name) {
+            if (mpu_algorithm_checksum_name.len) {
                 struct aws_byte_cursor checksum = aws_byte_cursor_from_buf(&part->checksum_base64);
 
                 if (aws_byte_buf_append_dynamic(body_buffer, &s_open_start_bracket)) {
                     goto error_clean_up;
                 }
-                if (aws_byte_buf_append_dynamic(body_buffer, mpu_algorithm_checksum_name)) {
+                if (aws_byte_buf_append_dynamic(body_buffer, &mpu_algorithm_checksum_name)) {
                     goto error_clean_up;
                 }
                 if (aws_byte_buf_append_dynamic(body_buffer, &s_close_bracket)) {
@@ -659,7 +744,7 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
                 if (aws_byte_buf_append_dynamic(body_buffer, &s_open_end_bracket)) {
                     goto error_clean_up;
                 }
-                if (aws_byte_buf_append_dynamic(body_buffer, mpu_algorithm_checksum_name)) {
+                if (aws_byte_buf_append_dynamic(body_buffer, &mpu_algorithm_checksum_name)) {
                     goto error_clean_up;
                 }
                 if (aws_byte_buf_append_dynamic(body_buffer, &s_close_bracket_new_line)) {
@@ -675,6 +760,8 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
             goto error_clean_up;
         }
 
+        AWS_LOGF_TRACE(
+            AWS_LS_S3_GENERAL, "Payload for Complete MPU is:\n" PRInSTR "\n", AWS_BYTE_BUF_PRI(*body_buffer));
         aws_s3_message_util_assign_body(
             allocator, body_buffer, message, NULL /* checksum_config */, NULL /* out_checksum */);
     }
@@ -724,12 +811,100 @@ error_clean_up:
     return NULL;
 }
 
+/**
+ * Calculate the in memory checksum based on the checksum config. Initialize and set the out_checksum to the encoded
+ * checksum result
+ */
+static int s_calculate_in_memory_checksum_helper(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor data,
+    const struct checksum_config_storage *checksum_config,
+    struct aws_byte_buf *out_checksum) {
+    AWS_ASSERT(checksum_config->checksum_algorithm != AWS_SCA_NONE);
+    AWS_ASSERT(out_checksum != NULL);
+    AWS_ZERO_STRUCT(*out_checksum);
+
+    int ret_code = AWS_OP_ERR;
+    size_t digest_size = aws_get_digest_size_from_checksum_algorithm(checksum_config->checksum_algorithm);
+    size_t encoded_checksum_len = 0;
+    if (aws_base64_compute_encoded_len(digest_size, &encoded_checksum_len)) {
+        return AWS_OP_ERR;
+    }
+
+    aws_byte_buf_init(out_checksum, allocator, encoded_checksum_len);
+
+    struct aws_byte_buf raw_checksum;
+    aws_byte_buf_init(&raw_checksum, allocator, digest_size);
+
+    if (aws_checksum_compute(allocator, checksum_config->checksum_algorithm, &data, &raw_checksum)) {
+        goto done;
+    }
+    struct aws_byte_cursor raw_checksum_cursor = aws_byte_cursor_from_buf(&raw_checksum);
+    if (aws_base64_encode(&raw_checksum_cursor, out_checksum)) {
+        goto done;
+    }
+
+    ret_code = AWS_OP_SUCCESS;
+done:
+    if (ret_code) {
+        aws_byte_buf_clean_up(out_checksum);
+    }
+    aws_byte_buf_clean_up(&raw_checksum);
+    return ret_code;
+}
+
+/**
+ * Calculate the in memory checksum based on the checksum config.
+ * If out_checksum set, initialize and set it to the encoded checksum result.
+ * Set the corresponding header in out_message to the encoded checksum result.
+ */
+static int s_calculate_and_add_checksum_to_header_helper(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor data,
+    const struct checksum_config_storage *checksum_config,
+    struct aws_http_message *out_message,
+    struct aws_byte_buf *out_checksum) {
+    AWS_ASSERT(checksum_config->checksum_algorithm != AWS_SCA_NONE);
+    AWS_ASSERT(out_message != NULL);
+    int ret_code = AWS_OP_ERR;
+
+    struct aws_byte_buf local_encoded_checksum_buf;
+    struct aws_byte_buf *local_encoded_checksum;
+    if (out_checksum == NULL) {
+        local_encoded_checksum = &local_encoded_checksum_buf;
+    } else {
+        local_encoded_checksum = out_checksum;
+    }
+    AWS_ZERO_STRUCT(*local_encoded_checksum);
+    if (s_calculate_in_memory_checksum_helper(allocator, data, checksum_config, local_encoded_checksum)) {
+        goto done;
+    }
+
+    /* Add the encoded checksum to header. */
+    const struct aws_byte_cursor header_name =
+        aws_get_http_header_name_from_checksum_algorithm(checksum_config->checksum_algorithm);
+    struct aws_byte_cursor encoded_checksum_val = aws_byte_cursor_from_buf(local_encoded_checksum);
+    struct aws_http_headers *headers = aws_http_message_get_headers(out_message);
+    if (aws_http_headers_set(headers, header_name, encoded_checksum_val)) {
+        goto done;
+    }
+
+    ret_code = AWS_OP_SUCCESS;
+done:
+    if (ret_code || out_checksum == NULL) {
+        /* In case of error happen or out_checksum is not set, clean up the encoded checksum. Otherwise, the caller will
+         * own the encoded checksum. */
+        aws_byte_buf_clean_up(local_encoded_checksum);
+    }
+    return ret_code;
+}
+
 /* Assign a buffer to an HTTP message, creating a stream and setting the content-length header */
 struct aws_input_stream *aws_s3_message_util_assign_body(
     struct aws_allocator *allocator,
     struct aws_byte_buf *byte_buf,
     struct aws_http_message *out_message,
-    const struct checksum_config *checksum_config,
+    const struct checksum_config_storage *checksum_config,
     struct aws_byte_buf *out_checksum) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(out_message);
@@ -783,7 +958,7 @@ struct aws_input_stream *aws_s3_message_util_assign_body(
             if (aws_http_headers_set(
                     headers,
                     g_trailer_header_name,
-                    *aws_get_http_header_name_from_algorithm(checksum_config->checksum_algorithm))) {
+                    aws_get_http_header_name_from_checksum_algorithm(checksum_config->checksum_algorithm))) {
                 goto error_clean_up;
             }
             /* set x-amz-decoded-content-length header */
@@ -806,6 +981,19 @@ struct aws_input_stream *aws_s3_message_util_assign_body(
             }
             aws_input_stream_release(input_stream);
             input_stream = chunk_stream;
+        } else if (checksum_config->location == AWS_SCL_HEADER) {
+            /* Calculate the checksum directly from memory and add it to the header. */
+            if (s_calculate_and_add_checksum_to_header_helper(
+                    allocator, buffer_byte_cursor, checksum_config, out_message, out_checksum)) {
+                goto error_clean_up;
+            }
+
+        } else if (checksum_config->checksum_algorithm != AWS_SCA_NONE && out_checksum != NULL) {
+            /* In case checksums still wanted, and we can calculate it directly from the buffer in memory to
+             * out_checksum */
+            if (s_calculate_in_memory_checksum_helper(allocator, buffer_byte_cursor, checksum_config, out_checksum)) {
+                goto error_clean_up;
+            }
         }
     }
     int64_t stream_length = 0;
@@ -833,17 +1021,6 @@ error_clean_up:
     return NULL;
 }
 
-bool aws_s3_message_util_check_checksum_header(struct aws_http_message *message) {
-    struct aws_http_headers *headers = aws_http_message_get_headers(message);
-    for (int algorithm = AWS_SCA_INIT; algorithm <= AWS_SCA_END; algorithm++) {
-        const struct aws_byte_cursor *algorithm_header_name = aws_get_http_header_name_from_algorithm(algorithm);
-        if (aws_http_headers_has(headers, *algorithm_header_name)) {
-            return true;
-        }
-    }
-    return false;
-}
-
 /* Add a content-md5 header. */
 int aws_s3_message_util_add_content_md5_header(
     struct aws_allocator *allocator,
@@ -867,9 +1044,7 @@ int aws_s3_message_util_add_content_md5_header(
         return AWS_OP_ERR;
     }
     struct aws_byte_buf base64_output_buf;
-    if (aws_byte_buf_init(&base64_output_buf, allocator, base64_output_size)) {
-        return AWS_OP_ERR;
-    }
+    aws_byte_buf_init(&base64_output_buf, allocator, base64_output_size);
     if (aws_base64_encode(&base64_input, &base64_output_buf)) {
         goto error_clean_up;
     }
@@ -1031,9 +1206,7 @@ int aws_s3_message_util_set_multipart_request_path(
         return AWS_OP_ERR;
     }
 
-    if (aws_byte_buf_init(&request_path_buf, allocator, request_path.len)) {
-        return AWS_OP_ERR;
-    }
+    aws_byte_buf_init(&request_path_buf, allocator, request_path.len);
 
     if (aws_byte_buf_append_dynamic(&request_path_buf, &request_path)) {
         goto error_clean_up;
