@@ -34,17 +34,6 @@ AWS_SDKUTILS_API struct aws_owning_cursor aws_endpoints_non_owning_cursor_create
 /* Cleans up memory associated with the cursor */
 AWS_SDKUTILS_API void aws_owning_cursor_clean_up(struct aws_owning_cursor *cursor);
 
-/*
- * Determine whether host cursor is IPv4 string.
- */
-AWS_SDKUTILS_API bool aws_is_ipv4(struct aws_byte_cursor host);
-
-/*
- * Determine whether host cursor is IPv6 string.
- * Supports checking for uri encoded strings and scoped literals.
- */
-AWS_SDKUTILS_API bool aws_is_ipv6(struct aws_byte_cursor host, bool is_uri_encoded);
-
 /*
  * Determine whether label is a valid host label.
  */
@@ -85,7 +74,7 @@ AWS_SDKUTILS_API void aws_array_list_deep_clean_up(
 
 /* Function that resolves template. */
 typedef int(aws_endpoints_template_resolve_fn)(
-    struct aws_byte_cursor template,
+    struct aws_byte_cursor template_cursor,
     void *user_data,
     struct aws_owning_cursor *out_resolved);
 /*
