@@ -81,11 +81,11 @@ bool s2n_hmac_is_available(s2n_hmac_algorithm hmac_alg)
     switch(hmac_alg) {
     case S2N_HMAC_MD5:
     case S2N_HMAC_SSLv3_MD5:
+         /* Some libcryptos, such as OpenSSL, disable MD5 by default when in FIPS mode, which is 
+          * required in order to negotiate SSLv3. However, this is supported in AWS-LC. 
+          */ 
+         return !s2n_is_in_fips_mode() || s2n_libcrypto_is_awslc(); 
     case S2N_HMAC_SSLv3_SHA1:
-        /* Some libcryptos, such as OpenSSL, disable MD5 by default when in FIPS mode, which is
-         * required in order to negotiate SSLv3. However, this is supported in AWS-LC.
-         */
-        return !s2n_is_in_fips_mode() || s2n_libcrypto_is_awslc();
     case S2N_HMAC_NONE:
     case S2N_HMAC_SHA1:
     case S2N_HMAC_SHA224:
@@ -332,7 +332,7 @@ int s2n_hmac_reset(struct s2n_hmac_state *state)
     POSIX_ENSURE(state->hash_block_size != 0, S2N_ERR_PRECONDITION_VIOLATION);
     POSIX_GUARD(s2n_hash_copy(&state->inner, &state->inner_just_key));
 
-    uint64_t bytes_in_hash;
+    uint64_t bytes_in_hash = 0;
     POSIX_GUARD(s2n_hash_get_currently_in_hash_total(&state->inner, &bytes_in_hash));
     bytes_in_hash %= state->hash_block_size;
     POSIX_ENSURE(bytes_in_hash <= UINT32_MAX, S2N_ERR_INTEGER_OVERFLOW);
