@@ -50,12 +50,7 @@
     #define RESULT_EVP_CTX_INIT(ctx) EVP_CIPHER_CTX_init(ctx)
 #endif
 
-#if !defined(OPENSSL_IS_BORINGSSL) && !defined(OPENSSL_FIPS) && !defined(LIBRESSL_VERSION_NUMBER) && !defined(OPENSSL_IS_AWSLC) && !defined(OPENSSL_NO_ENGINE)
-    #define S2N_LIBCRYPTO_SUPPORTS_CUSTOM_RAND 1
-#else
-    #define S2N_LIBCRYPTO_SUPPORTS_CUSTOM_RAND 0
-#endif
-
+bool s2n_libcrypto_is_openssl(void);
 bool s2n_libcrypto_is_awslc();
 bool s2n_libcrypto_is_boringssl();
 bool s2n_libcrypto_is_libressl();
