@@ -17,12 +17,12 @@
 
 #include "crypto/s2n_openssl.h"
 
-bool s2n_libcrypto_supports_kyber()
+bool s2n_libcrypto_supports_evp_kem()
 {
-    /* S2N_LIBCRYPTO_SUPPORTS_KYBER will be auto-detected and #defined if
-     * ./tests/features/S2N_LIBCRYPTO_SUPPORTS_KYBER.c successfully compiles
+    /* S2N_LIBCRYPTO_SUPPORTS_EVP_KEM will be auto-detected and #defined if
+     * ./tests/features/S2N_LIBCRYPTO_SUPPORTS_EVP_KEM.c successfully compiles
      */
-#if defined(S2N_LIBCRYPTO_SUPPORTS_KYBER)
+#if defined(S2N_LIBCRYPTO_SUPPORTS_EVP_KEM)
     return true;
 #else
     return false;
@@ -31,5 +31,17 @@ bool s2n_libcrypto_supports_kyber()
 
 bool s2n_pq_is_enabled()
 {
-    return s2n_libcrypto_supports_kyber();
+    return s2n_libcrypto_supports_evp_kem();
+}
+
+bool s2n_libcrypto_supports_mlkem()
+{
+    /* S2N_LIBCRYPTO_SUPPORTS_MLKEM will be auto-detected and #defined if
+     * ./tests/features/S2N_LIBCRYPTO_SUPPORTS_MLKEM.c successfully compiles
+     */
+#if defined(S2N_LIBCRYPTO_SUPPORTS_MLKEM)
+    return true;
+#else
+    return false;
+#endif
 }
