@@ -1,16 +1,19 @@
+# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# SPDX-License-Identifier: Apache-2.0
 import os
 import pytest
 import subprocess
 
-from global_flags import get_flag, is_criterion_on, S2N_USE_CRITERION
+from global_flags import get_flag
 from processes import ManagedProcess
-from providers import Provider, CriterionS2N, S2N
+from providers import Provider, S2N
 
 from common import ProviderOptions
+from conftest import PATH_CONFIGURATION_KEY
 
 
 @pytest.fixture
-def managed_process():
+def managed_process(request: pytest.FixtureRequest):
     """
     Generic process manager. This could be used to launch any process as a background
     task and cleanup when finished.
@@ -20,16 +23,27 @@ def managed_process():
     allows cleanup after a test, even if a failure occurred.
     """
     processes = []
+    # Indicates whether a launch was aborted. If so, non-graceful shutdown is allowed
+    aborted = False
 
     def _fn(provider_class: Provider, options: ProviderOptions, timeout=5, send_marker=None, close_marker=None,
             expect_stderr=None, kill_marker=None, send_with_newline=None):
-        if provider_class == S2N and is_criterion_on():
-            provider_class = CriterionS2N
-            # This comes from the number of iterations specific in the rust benchmark handler(s).
-            # currently set at 10 iterations, so give us 10x more time.
-            timeout = timeout * 10
+        best_effort_mode = request.config.getoption("--best-effort-NOT-FOR-CI")
+        if best_effort_mode:
+            # modify the `aborted` field in the generator object
+            nonlocal aborted
+            available_providers = request.config.stash[PATH_CONFIGURATION_KEY]
+            if provider_class not in available_providers:
+                aborted = True
+                pytest.skip(f"{provider_class} is not available")
+
         provider = provider_class(options)
         cmd_line = provider.get_cmd_line()
+
+        if best_effort_mode and provider_class is S2N and not (cmd_line[0] == "s2nc" or cmd_line[0] == "s2nd"):
+            aborted = True
+            pytest.skip("s2nc_head or s2nd_head not supported for best-effort")
+
         # The process will default to send markers in the providers.py file
         # if not specified by a test.
         if send_marker is not None:
@@ -70,7 +84,10 @@ def managed_process():
     finally:
         # Whether the processes succeeded or not, clean then up.
         for p in processes:
-            p.join()
+            if aborted:
+                p.kill()
+            else:
+                p.join()
 
 
 def _swap_mtu(device, new_mtu):
