@@ -1,3 +1,5 @@
+# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# SPDX-License-Identifier: Apache-2.0
 import os
 import select
 import selectors
@@ -173,13 +175,23 @@ class _processCommunicator(object):
                                 input_data_offset = 0
                                 if send_marker_list:
                                     send_marker = send_marker_list.pop(0)
-                                print(f'{self.name}: next send_marker is {send_marker}')
+                                    print(f'{self.name}: next send_marker is {send_marker}')
                     elif key.fileobj in (self.proc.stdout, self.proc.stderr):
                         print(f'{self.name}: stdout available')
                         data = os.read(key.fd, 32768)
                         if not data:
                             selector.unregister(key.fileobj)
                         data_str = str(data)
+
+                        # Prepends n - 1 bytes of previously-seen stdout to the chunk we'll be searching
+                        # through, where n is the size of the send_marker we're currently looking for.
+                        # This ensures a marker doesn't get split between chunks and we miss it.
+                        if self._fileobj2output[key.fileobj] and send_marker:
+                            stored_stdout_list = self._fileobj2output[key.fileobj]
+                            send_marker_len = len(send_marker) - 1
+                            if len(stored_stdout_list) > 0:
+                                data_str = str(stored_stdout_list[-1][-send_marker_len:] + data)
+
                         data_debug = data_str[:_DEBUG_LEN]
                         if len(data_str) > _DEBUG_LEN:
                             data_debug += f' ...({len(data_str) - _DEBUG_LEN} more bytes)'
@@ -195,6 +207,7 @@ class _processCommunicator(object):
                             print(f'{self.name}: looking for send_marker {send_marker} in {data_debug}')
                         if send_marker is not None and send_marker in data_str:
                             print(f'{self.name}: found {send_marker}')
+                            send_marker = None
                             if self.proc.stdin and input_data:
                                 selector.register(
                                     self.proc.stdin, selectors.EVENT_WRITE)
