@@ -1,3 +1,5 @@
+# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# SPDX-License-Identifier: Apache-2.0
 import os
 import subprocess
 import pytest
@@ -5,7 +7,6 @@ import threading
 
 from common import ProviderOptions, Ciphers, Curves, Protocols, Signatures
 from global_flags import get_flag, S2N_PROVIDER_VERSION, S2N_FIPS_MODE
-from global_flags import S2N_USE_CRITERION
 from stat import S_IMODE
 
 
@@ -147,22 +148,24 @@ class S2N(Provider):
 
     @classmethod
     def supports_protocol(cls, protocol, with_cert=None):
-        # TLS 1.3 is unsupported for openssl-1.0
+        # RSA-PSS is unsupported for openssl-1.0
         # libressl and boringssl are disabled because of configuration issues
         # see https://github.com/aws/s2n-tls/issues/3250
-        TLS_13_UNSUPPORTED_LIBCRYPTOS = {
+        PSS_UNSUPPORTED_LIBCRYPTOS = {
             "libressl",
             "boringssl",
             "openssl-1.0"
         }
-
-        # Disable TLS 1.3 tests for all libcryptos that don't support 1.3
-        if protocol == Protocols.TLS13:
-            current_libcrypto = get_flag(S2N_PROVIDER_VERSION)
-            for unsupported_lc in TLS_13_UNSUPPORTED_LIBCRYPTOS:
-                # e.g. "openssl-1.0" in "openssl-1.0.2-fips"
-                if unsupported_lc in current_libcrypto:
-                    return False
+        pss_is_unsupported = any([
+            # e.g. "openssl-1.0" in "openssl-1.0.2-fips"
+            libcrypto in get_flag(S2N_PROVIDER_VERSION)
+            for libcrypto in PSS_UNSUPPORTED_LIBCRYPTOS
+        ])
+        if pss_is_unsupported:
+            if protocol == Protocols.TLS13:
+                return False
+            if with_cert and with_cert.algorithm == 'RSAPSS':
+                return False
 
         # SSLv3 cannot be negotiated in FIPS mode with libcryptos other than AWS-LC.
         if all([
@@ -330,95 +333,7 @@ class S2N(Provider):
         return cmd_line
 
 
-class CriterionS2N(S2N):
-    """
-    Wrap the S2N provider in criterion-rust
-    The CriterionS2N provider modifies the test command being run by pytest to be the criterion benchmark binary
-    and moves the s2nc/d command line arguments into an environment variable.  The binary created by
-    `cargo bench --no-run` has a unique name and must be searched for, which the CriterionS2N provider finds
-    and replaces in the final testing command. The arguments to s2nc/d are moved to the environment variables
-    `S2NC_ARGS` or `S2ND_ARGS`, along with the test name, which are read by the rust benchmark when spawning
-    s2nc/d as subprocesses.
-    """
-    criterion_off = 'off'
-    criterion_delta = 'delta'
-    criterion_baseline = 'baseline'
-    # Figure out what mode to run in: baseline or delta
-    criterion_mode = get_flag(S2N_USE_CRITERION)
-
-    def _find_s2n_benchmark(self, pattern):
-        # Use executable bit to find the correct file.
-        result = find_files(pattern, root_dir=self.cargo_root, modes=['0o775', '0o755'])
-        if len(result) != 1:
-            raise FileNotFoundError(
-                f"Exactly one s2n criterion benchmark not found. Found {result}.")
-        else:
-            return result[0]
-
-    def _find_cargo(self):
-        return os.path.abspath("../../bindings/rust")
-
-    def _cargo_bench(self):
-        """
-        Find the handlers
-        """
-        self.s2nc_bench = self._find_s2n_benchmark("s2nc-")
-        self.s2nd_bench = self._find_s2n_benchmark("s2nd-")
-
-    def __init__(self, options: ProviderOptions):
-        super().__init__(options)
-        # Set cargo root
-        self.cargo_root = self._find_cargo()
-
-        # Find the criterion binaries
-        self._cargo_bench()
-
-        # strip off the s2nc/d at the front because criterion
-        if 's2nc' in self.cmd_line[0] or 's2nd' in self.cmd_line[0]:
-            self.cmd_line = self.cmd_line[1:]
-
-        # strip off the s2nc -e argument, criterion handler isn't sending any STDIN characters,
-        # and makes it look like s2nc is hanging.
-        # WARNING: this is a blocker for any test that requires STDIN.
-        if '-e' in self.cmd_line:
-            self.cmd_line.remove('-e')
-            print(f"***** cmd_line is now {self.cmd_line}")
-
-        # Copy the command arguments to an environment variable for the harness to read.
-        if self.options.mode == Provider.ServerMode:
-            self.options.env_overrides.update(
-                {'S2ND_ARGS': ' '.join(self.cmd_line),
-                 'S2ND_TEST_NAME': f"{self.options.cipher}_{self.options.host}"})
-            self.capture_server_args()
-        elif self.options.mode == Provider.ClientMode:
-            self.options.env_overrides.update(
-                {'S2NC_ARGS': ' '.join(self.cmd_line),
-                 'S2NC_TEST_NAME': f"{self.options.cipher}_{self.options.host}"})
-            if self.criterion_mode == CriterionS2N.criterion_baseline:
-                self.capture_client_args_baseline()
-            if self.criterion_mode == CriterionS2N.criterion_delta:
-                self.capture_client_args_delta()
-
-    def capture_server_args(self):
-        self.cmd_line = [self.s2nd_bench, "--bench",
-                         "s2nd", "--save-baseline", "main"]
-
-    # Saves baseline data with the tag "main"
-    # see https://bheisler.github.io/criterion.rs/book/user_guide/command_line_options.html
-    def capture_client_args_baseline(self):
-        self.cmd_line = [self.s2nc_bench, "--bench",
-                         "s2nc", "--save-baseline", "main"]
-
-    # "By default, Criterion.rs will compare the measurements against the previous run"
-    # This run is stored with the tag "new"
-    # https://bheisler.github.io/criterion.rs/book/user_guide/command_line_options.html
-    def capture_client_args_delta(self):
-        self.cmd_line = [self.s2nc_bench, "--bench", "s2nc", "--plotting-backend", "plotters", "--baseline", "main"]
-
-
 class OpenSSL(Provider):
-    _version = get_flag(S2N_PROVIDER_VERSION)
-
     def __init__(self, options: ProviderOptions):
         Provider.__init__(self, options)
         # We print some OpenSSL logging that includes stderr
@@ -473,7 +388,7 @@ class OpenSSL(Provider):
 
     @classmethod
     def get_version(cls):
-        return cls._version
+        return get_flag(S2N_PROVIDER_VERSION)
 
     @classmethod
     def supports_protocol(cls, protocol, with_cert=None):
