@@ -12,6 +12,12 @@
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
+
+#ifndef _S2N_PRELUDE_INCLUDED
+    /* make sure s2n_prelude.h is includes as part of the compiler flags, if not then fail the build */
+    #error "Expected s2n_prelude.h to be included as part of the compiler flags"
+#endif
+
 #define _GNU_SOURCE
 #include <ctype.h>
 #include <openssl/crypto.h>
@@ -59,6 +65,42 @@ S2N_RESULT s2n_test_lowercase_copy(const char *input, char *destination, size_t
     return S2N_RESULT_OK;
 }
 
+S2N_RESULT s2n_check_supported_libcrypto(const char *s2n_libcrypto)
+{
+    RESULT_ENSURE_REF(s2n_libcrypto);
+
+    /* List of supported libcrypto variants we test with */
+    const struct {
+        const char *libcrypto;
+        bool is_openssl;
+    } supported_libcrypto[] = {
+        { .libcrypto = "awslc", .is_openssl = false },
+        { .libcrypto = "awslc-fips", .is_openssl = false },
+        { .libcrypto = "awslc-fips-2022", .is_openssl = false },
+        { .libcrypto = "boringssl", .is_openssl = false },
+        { .libcrypto = "libressl", .is_openssl = false },
+        { .libcrypto = "openssl-1.0.2", .is_openssl = true },
+        { .libcrypto = "openssl-1.1.1", .is_openssl = true },
+        { .libcrypto = "openssl-3.0", .is_openssl = true },
+    };
+
+    for (size_t i = 0; i < s2n_array_len(supported_libcrypto); i++) {
+        /* The linked libcryto is one of the known supported libcrypto variants */
+        if (strcmp(s2n_libcrypto, supported_libcrypto[i].libcrypto) == 0) {
+            if (supported_libcrypto[i].is_openssl) {
+                EXPECT_TRUE(s2n_libcrypto_is_openssl());
+            } else {
+                EXPECT_FALSE(s2n_libcrypto_is_openssl());
+            }
+
+            return S2N_RESULT_OK;
+        }
+    }
+
+    /* Testing with an unexpected libcrypto. */
+    return S2N_RESULT_ERROR;
+}
+
 int main()
 {
     BEGIN_TEST();
@@ -84,6 +126,18 @@ int main()
         END_TEST();
     }
 
+    /* Ensure that FIPS mode is enabled when linked to AWS-LC-FIPS, and disabled when linked to AWS-LC */
+    if (strstr(s2n_libcrypto, "awslc") != NULL) {
+        s2n_fips_mode fips_mode = S2N_FIPS_MODE_DISABLED;
+        EXPECT_SUCCESS(s2n_get_fips_mode(&fips_mode));
+
+        if (strstr(s2n_libcrypto, "fips") != NULL) {
+            EXPECT_EQUAL(fips_mode, S2N_FIPS_MODE_ENABLED);
+        } else {
+            EXPECT_EQUAL(fips_mode, S2N_FIPS_MODE_DISABLED);
+        }
+    }
+
     char s2n_libcrypto_copy[MAX_LIBCRYPTO_NAME_LEN] = { 0 };
     EXPECT_TRUE(strlen(s2n_libcrypto) < MAX_LIBCRYPTO_NAME_LEN);
     EXPECT_OK(s2n_test_lowercase_copy(s2n_libcrypto, &s2n_libcrypto_copy[0], s2n_array_len(s2n_libcrypto_copy)));
@@ -113,6 +167,15 @@ int main()
         }
     };
 
+    /* Ensure we are testing with supported libcryto variants.
+     *
+     * We need to update s2n_libcrypto_is_openssl() when adding support
+     * for a new libcrypto.
+     */
+    {
+        EXPECT_OK(s2n_check_supported_libcrypto(s2n_libcrypto));
+    };
+
     END_TEST();
     return 0;
 }
