@@ -21,8 +21,3 @@
 
 extern const s2n_extension_type s2n_client_alpn_extension;
 bool s2n_client_alpn_should_send(struct s2n_connection *conn);
-
-/* Old-style extension functions -- remove after extensions refactor is complete */
-
-extern int s2n_extensions_client_alpn_send(struct s2n_connection *conn, struct s2n_stuffer *out);
-extern int s2n_recv_client_alpn(struct s2n_connection *conn, struct s2n_stuffer *extension);
