@@ -26,6 +26,7 @@
 #include "crypto/s2n_rsa_signing.h"
 #include "error/s2n_errno.h"
 #include "stuffer/s2n_stuffer.h"
+#include "tls/extensions/s2n_client_server_name.h"
 #include "tls/extensions/s2n_client_supported_groups.h"
 #include "tls/extensions/s2n_extension_list.h"
 #include "tls/extensions/s2n_server_key_share.h"
@@ -84,7 +85,7 @@ static S2N_RESULT s2n_generate_client_session_id(struct s2n_connection *conn)
 
     /* QUIC should not allow session ids for any reason.
      *
-     *= https://tools.ietf.org/rfc/rfc9001#section-8.4
+     *= https://www.rfc-editor.org/rfc/rfc9001#section-8.4
      *# A server SHOULD treat the receipt of a TLS ClientHello with a non-empty
      *# legacy_session_id field as a connection error of type PROTOCOL_VIOLATION.
      */
@@ -224,46 +225,47 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
     }
 
     /*
-     *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+     *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
      *# The client will also send a
      *# ClientHello when the server has responded to its ClientHello with a
      *# HelloRetryRequest.  In that case, the client MUST send the same
      *# ClientHello without modification, except as follows:
      *
      * All of the exceptions that follow are extensions.
-     * Ignoring the extensions, the client hellos should match /exactly/.
      */
-    ssize_t old_msg_len = old_ch->raw_message.size;
-    /* Also consider the 2-byte size of the extension list */
-    ssize_t old_extensions_len = old_ch->extensions.raw.size + sizeof(uint16_t);
-    RESULT_ENSURE_GT(old_msg_len, old_extensions_len);
-    size_t verify_len = old_msg_len - old_extensions_len;
-    RESULT_ENSURE_LTE(verify_len, new_ch->raw_message.size);
-    RESULT_ENSURE(s2n_constant_time_equals(
-                          old_ch->raw_message.data,
-                          new_ch->raw_message.data,
-                          verify_len),
+    RESULT_ENSURE(old_ch->legacy_version == new_ch->legacy_version, S2N_ERR_BAD_MESSAGE);
+    RESULT_ENSURE(old_ch->compression_methods.size == new_ch->compression_methods.size, S2N_ERR_BAD_MESSAGE);
+    RESULT_ENSURE(s2n_constant_time_equals(old_ch->compression_methods.data, new_ch->compression_methods.data,
+                          new_ch->compression_methods.size),
             S2N_ERR_BAD_MESSAGE);
 
-    /* In the past, the s2n-tls client updated the client hello in ways not
-     * allowed by RFC8446: https://github.com/aws/s2n-tls/pull/3311
-     * Although the issue was addressed, its existence means that old versions
-     * of the s2n-tls client will fail this validation.
-     *
-     * So to avoid breaking old s2n-tls clients, we do not enforce this validation
-     * outside of tests. We continue to enforce it during tests to avoid regressions.
+    /* Some clients are not compliant with TLS 1.3 RFC, and send mismatching values in their second
+     * ClientHello. For increased compatibility, these checks are skipped outside of tests. The
+     * checks are still included in tests to ensure the s2n-tls client remains compliant.
      */
     if (s2n_in_test()) {
-        /*
-         * We need to verify the client random separately
-         * because we erase it from the client hello during parsing.
-         * Compare the old value to the current value.
+        /* In the past, the s2n-tls client updated the client random in the second ClientHello
+         * which is not allowed by RFC8446: https://github.com/aws/s2n-tls/pull/3311. Although the
+         * issue was addressed, its existence means that old versions of the s2n-tls client will
+         * fail this validation.
          */
         RESULT_ENSURE(s2n_constant_time_equals(
                               previous_client_random,
                               conn->handshake_params.client_random,
                               S2N_TLS_RANDOM_DATA_LEN),
                 S2N_ERR_BAD_MESSAGE);
+
+        /* Some clients have been found to send a mismatching legacy session ID. */
+        RESULT_ENSURE(old_ch->session_id.size == new_ch->session_id.size, S2N_ERR_BAD_MESSAGE);
+        RESULT_ENSURE(s2n_constant_time_equals(old_ch->session_id.data, new_ch->session_id.data,
+                              new_ch->session_id.size),
+                S2N_ERR_BAD_MESSAGE);
+
+        /* Some clients have been found to send a mismatching cipher suite list. */
+        RESULT_ENSURE(old_ch->cipher_suites.size == new_ch->cipher_suites.size, S2N_ERR_BAD_MESSAGE);
+        RESULT_ENSURE(s2n_constant_time_equals(old_ch->cipher_suites.data, new_ch->cipher_suites.data,
+                              new_ch->cipher_suites.size),
+                S2N_ERR_BAD_MESSAGE);
     }
 
     /*
@@ -290,7 +292,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
 
         switch (extension_type) {
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  If a "key_share" extension was supplied in the HelloRetryRequest,
              *#       replacing the list of shares with a list containing a single
              *#       KeyShareEntry from the indicated group.
@@ -299,7 +301,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
                 /* Handled when parsing the key share extension */
                 break;
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  Removing the "early_data" extension (Section 4.2.10) if one was
              *#       present.  Early data is not permitted after a HelloRetryRequest.
              */
@@ -307,7 +309,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
                 RESULT_ENSURE(new_size == 0, S2N_ERR_BAD_MESSAGE);
                 break;
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  Including a "cookie" extension if one was provided in the
              *#       HelloRetryRequest.
              */
@@ -315,7 +317,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
                 /* Handled when parsing the cookie extension */
                 break;
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  Updating the "pre_shared_key" extension if present by recomputing
              *#       the "obfuscated_ticket_age" and binder values and (optionally)
              *#       removing any PSKs which are incompatible with the server's
@@ -324,6 +326,17 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
             case TLS_EXTENSION_PRE_SHARED_KEY:
                 /* Handled when parsing the psk extension */
                 break;
+
+            /* Some clients have been found to send mismatching supported versions in their second
+             * ClientHello. The extension isn't compared byte-for-byte for increased compatibility
+             * with these clients.
+             */
+            case TLS_EXTENSION_SUPPORTED_VERSIONS:
+                /* Additional HRR validation for the supported versions extension is performed when
+                 * parsing the extension.
+                 */
+                break;
+
             /*
              * No more exceptions.
              * All other extensions must match.
@@ -458,34 +471,6 @@ int s2n_parse_client_hello(struct s2n_connection *conn)
     conn->session_id_len = conn->client_hello.session_id.size;
     POSIX_CHECKED_MEMCPY(conn->session_id, conn->client_hello.session_id.data, conn->session_id_len);
 
-    /* Set default key exchange curve.
-     * This is going to be our fallback if the client has no preference.
-     *
-     * P-256 is our preferred fallback option because the TLS1.3 RFC requires
-     * all implementations to support it:
-     *
-     *     https://tools.ietf.org/rfc/rfc8446#section-9.1
-     *     A TLS-compliant application MUST support key exchange with secp256r1 (NIST P-256)
-     *     and SHOULD support key exchange with X25519 [RFC7748]
-     *
-     *= https://tools.ietf.org/rfc/rfc4492#section-4
-     *# A client that proposes ECC cipher suites may choose not to include these extensions.
-     *# In this case, the server is free to choose any one of the elliptic curves or point formats listed in Section 5.
-     *
-     */
-    const struct s2n_ecc_preferences *ecc_pref = NULL;
-    POSIX_GUARD(s2n_connection_get_ecc_preferences(conn, &ecc_pref));
-    POSIX_ENSURE_REF(ecc_pref);
-    POSIX_ENSURE_GT(ecc_pref->count, 0);
-    if (s2n_ecc_preferences_includes_curve(ecc_pref, TLS_EC_CURVE_SECP_256_R1)) {
-        conn->kex_params.server_ecc_evp_params.negotiated_curve = &s2n_ecc_curve_secp256r1;
-    } else {
-        /* If P-256 isn't allowed by the current security policy, instead choose
-         * the first / most preferred curve.
-         */
-        conn->kex_params.server_ecc_evp_params.negotiated_curve = ecc_pref->ecc_curves[0];
-    }
-
     POSIX_GUARD_RESULT(s2n_client_hello_verify_for_retry(conn,
             &previous_hello_retry, &conn->client_hello, previous_client_random));
     return S2N_SUCCESS;
@@ -544,7 +529,7 @@ int s2n_process_client_hello(struct s2n_connection *conn)
      * Negotiate protocol version, cipher suite, ALPN, select a cert, etc. */
     struct s2n_client_hello *client_hello = &conn->client_hello;
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     if (!s2n_connection_supports_tls13(conn) || !s2n_security_policy_supports_tls13(security_policy)) {
@@ -552,6 +537,34 @@ int s2n_process_client_hello(struct s2n_connection *conn)
         conn->actual_protocol_version = MIN(conn->server_protocol_version, S2N_TLS12);
     }
 
+    /* Set default key exchange curve.
+     * This is going to be our fallback if the client has no preference.
+     *
+     * P-256 is our preferred fallback option because the TLS1.3 RFC requires
+     * all implementations to support it:
+     *
+     *     https://tools.ietf.org/rfc/rfc8446#section-9.1
+     *     A TLS-compliant application MUST support key exchange with secp256r1 (NIST P-256)
+     *     and SHOULD support key exchange with X25519 [RFC7748]
+     *
+     *= https://www.rfc-editor.org/rfc/rfc4492#section-4
+     *# A client that proposes ECC cipher suites may choose not to include these extensions.
+     *# In this case, the server is free to choose any one of the elliptic curves or point formats listed in Section 5.
+     *
+     */
+    const struct s2n_ecc_preferences *ecc_pref = NULL;
+    POSIX_GUARD(s2n_connection_get_ecc_preferences(conn, &ecc_pref));
+    POSIX_ENSURE_REF(ecc_pref);
+    POSIX_ENSURE_GT(ecc_pref->count, 0);
+    if (s2n_ecc_preferences_includes_curve(ecc_pref, TLS_EC_CURVE_SECP_256_R1)) {
+        conn->kex_params.server_ecc_evp_params.negotiated_curve = &s2n_ecc_curve_secp256r1;
+    } else {
+        /* If P-256 isn't allowed by the current security policy, instead choose
+         * the first / most preferred curve.
+         */
+        conn->kex_params.server_ecc_evp_params.negotiated_curve = ecc_pref->ecc_curves[0];
+    }
+
     POSIX_GUARD(s2n_extension_list_process(S2N_EXTENSION_LIST_CLIENT_HELLO, conn, &conn->client_hello.extensions));
 
     /* After parsing extensions, select a curve and corresponding keyshare to use */
@@ -581,12 +594,13 @@ int s2n_process_client_hello(struct s2n_connection *conn)
     POSIX_CHECKED_MEMCPY(previous_cipher_suite_iana, conn->secure->cipher_suite->iana_value, S2N_TLS_CIPHER_SUITE_LEN);
 
     /* Now choose the ciphers we have certs for. */
-    POSIX_GUARD(s2n_set_cipher_as_tls_server(conn, client_hello->cipher_suites.data, client_hello->cipher_suites.size / 2));
+    POSIX_GUARD(s2n_set_cipher_as_tls_server(conn, client_hello->cipher_suites.data,
+            client_hello->cipher_suites.size / 2));
 
     /* Check if this is the second client hello in a hello retry handshake */
     if (s2n_is_hello_retry_handshake(conn) && conn->handshake.message_number > 0) {
         /**
-         *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
          *# Servers MUST ensure that they negotiate the
          *# same cipher suite when receiving a conformant updated ClientHello (if
          *# the server selects the cipher suite as the first step in the
@@ -658,6 +672,12 @@ int s2n_client_hello_recv(struct s2n_connection *conn)
         /* Mark the client hello callback as invoked to avoid calling it again. */
         conn->client_hello.callback_invoked = true;
 
+        /* Do NOT move this null check. A test exists to assert that a server connection can get
+         * as far as the client hello callback without using its config. To do this we need a
+         * specific error for a null config just before the client hello callback. The test's
+         * assertions are weakened if this check is moved. */
+        POSIX_ENSURE(conn->config, S2N_ERR_CONFIG_NULL_BEFORE_CH_CALLBACK);
+
         /* Call client_hello_cb if exists, letting application to modify s2n_connection or swap s2n_config */
         if (conn->config->client_hello_cb) {
             int rc = conn->config->client_hello_cb(conn, conn->config->client_hello_cb_ctx);
@@ -688,7 +708,7 @@ int s2n_client_hello_send(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     const struct s2n_cipher_preferences *cipher_preferences = security_policy->cipher_preferences;
@@ -743,7 +763,7 @@ int s2n_client_hello_send(struct s2n_connection *conn)
 
     /**
      * For initial handshakes:
-     *= https://tools.ietf.org/rfc/rfc5746#3.4
+     *= https://www.rfc-editor.org/rfc/rfc5746#3.4
      *# o  The client MUST include either an empty "renegotiation_info"
      *#    extension, or the TLS_EMPTY_RENEGOTIATION_INFO_SCSV signaling
      *#    cipher suite value in the ClientHello.  Including both is NOT
@@ -752,7 +772,7 @@ int s2n_client_hello_send(struct s2n_connection *conn)
      * rather than the "renegotiation_info" extension.
      *
      * For renegotiation handshakes:
-     *= https://tools.ietf.org/rfc/rfc5746#3.5
+     *= https://www.rfc-editor.org/rfc/rfc5746#3.5
      *# The SCSV MUST NOT be included.
      */
     if (tls12_is_possible && !s2n_handshake_is_renegotiation(conn)) {
@@ -761,7 +781,10 @@ int s2n_client_hello_send(struct s2n_connection *conn)
     }
 
     /* Write size of the list of available ciphers */
-    POSIX_GUARD(s2n_stuffer_write_vector_size(&available_cipher_suites_size));
+    uint32_t ciphers_size = 0;
+    POSIX_GUARD(s2n_stuffer_get_vector_size(&available_cipher_suites_size, &ciphers_size));
+    POSIX_ENSURE(ciphers_size > 0, S2N_ERR_INVALID_CIPHER_PREFERENCES);
+    POSIX_GUARD(s2n_stuffer_write_reservation(&available_cipher_suites_size, ciphers_size));
 
     /* Zero compression methods */
     POSIX_GUARD(s2n_stuffer_write_uint8(out, 1));
@@ -808,7 +831,7 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_skip_write(&in_stuffer, client_hello->raw_message.size));
     struct s2n_stuffer *in = &in_stuffer;
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     if (conn->client_protocol_version < security_policy->minimum_protocol_version) {
@@ -818,15 +841,15 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     conn->actual_protocol_version = MIN(conn->client_protocol_version, conn->server_protocol_version);
 
     /* We start 5 bytes into the record */
-    uint16_t cipher_suites_length;
+    uint16_t cipher_suites_length = 0;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &cipher_suites_length));
     POSIX_ENSURE(cipher_suites_length > 0, S2N_ERR_BAD_MESSAGE);
     POSIX_ENSURE(cipher_suites_length % S2N_SSLv2_CIPHER_SUITE_LEN == 0, S2N_ERR_BAD_MESSAGE);
 
-    uint16_t session_id_length;
+    uint16_t session_id_length = 0;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &session_id_length));
 
-    uint16_t challenge_length;
+    uint16_t challenge_length = 0;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &challenge_length));
 
     S2N_ERROR_IF(challenge_length > S2N_TLS_RANDOM_DATA_LEN, S2N_ERR_BAD_MESSAGE);
@@ -838,7 +861,8 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     /* Find potential certificate matches before we choose the cipher. */
     POSIX_GUARD(s2n_conn_find_name_matching_certs(conn));
 
-    POSIX_GUARD(s2n_set_cipher_as_sslv2_server(conn, client_hello->cipher_suites.data, client_hello->cipher_suites.size / S2N_SSLv2_CIPHER_SUITE_LEN));
+    POSIX_GUARD(s2n_set_cipher_as_sslv2_server(conn, client_hello->cipher_suites.data,
+            client_hello->cipher_suites.size / S2N_SSLv2_CIPHER_SUITE_LEN));
     POSIX_GUARD_RESULT(s2n_signature_algorithm_select(conn));
     POSIX_GUARD(s2n_select_certs_for_server_auth(conn, &conn->handshake_params.our_chain_and_key));
 
@@ -866,7 +890,7 @@ int s2n_client_hello_get_parsed_extension(s2n_tls_extension_type extension_type,
     POSIX_ENSURE_REF(parsed_extension_list);
     POSIX_ENSURE_REF(parsed_extension);
 
-    s2n_extension_type_id extension_type_id;
+    s2n_extension_type_id extension_type_id = 0;
     POSIX_GUARD(s2n_extension_supported_iana_value_to_id(extension_type, &extension_type_id));
 
     s2n_parsed_extension *found_parsed_extension = &parsed_extension_list->parsed_extensions[extension_type_id];
@@ -962,7 +986,7 @@ int s2n_client_hello_get_legacy_record_version(struct s2n_client_hello *ch, uint
     return S2N_SUCCESS;
 }
 
-static S2N_RESULT s2n_client_hello_get_raw_extension(uint16_t extension_iana,
+S2N_RESULT s2n_client_hello_get_raw_extension(uint16_t extension_iana,
         struct s2n_blob *raw_extensions, struct s2n_blob *extension)
 {
     RESULT_ENSURE_REF(raw_extensions);
@@ -1046,3 +1070,47 @@ int s2n_client_hello_get_supported_groups(struct s2n_client_hello *ch, uint16_t
 
     return S2N_SUCCESS;
 }
+
+int s2n_client_hello_get_server_name_length(struct s2n_client_hello *ch, uint16_t *length)
+{
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(length);
+    *length = 0;
+
+    s2n_parsed_extension *server_name_extension = NULL;
+    POSIX_GUARD(s2n_client_hello_get_parsed_extension(S2N_EXTENSION_SERVER_NAME, &ch->extensions, &server_name_extension));
+    POSIX_ENSURE_REF(server_name_extension);
+
+    struct s2n_stuffer extension_stuffer = { 0 };
+    POSIX_GUARD(s2n_stuffer_init_written(&extension_stuffer, &server_name_extension->extension));
+
+    struct s2n_blob blob = { 0 };
+    POSIX_GUARD_RESULT(s2n_client_server_name_parse(&extension_stuffer, &blob));
+    *length = blob.size;
+
+    return S2N_SUCCESS;
+}
+
+int s2n_client_hello_get_server_name(struct s2n_client_hello *ch, uint8_t *server_name, uint16_t length, uint16_t *out_length)
+{
+    POSIX_ENSURE_REF(out_length);
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(server_name);
+    *out_length = 0;
+
+    s2n_parsed_extension *server_name_extension = NULL;
+    POSIX_GUARD(s2n_client_hello_get_parsed_extension(S2N_EXTENSION_SERVER_NAME, &ch->extensions, &server_name_extension));
+    POSIX_ENSURE_REF(server_name_extension);
+
+    struct s2n_stuffer extension_stuffer = { 0 };
+    POSIX_GUARD(s2n_stuffer_init_written(&extension_stuffer, &server_name_extension->extension));
+
+    struct s2n_blob blob = { 0 };
+    POSIX_GUARD_RESULT(s2n_client_server_name_parse(&extension_stuffer, &blob));
+    POSIX_ENSURE_LTE(blob.size, length);
+    POSIX_CHECKED_MEMCPY(server_name, blob.data, blob.size);
+
+    *out_length = blob.size;
+
+    return S2N_SUCCESS;
+}
