@@ -17,6 +17,7 @@
 
 #include <stdint.h>
 
+#include "tls/s2n_certificate_keys.h"
 #include "tls/s2n_cipher_preferences.h"
 #include "tls/s2n_ecc_preferences.h"
 #include "tls/s2n_kem_preferences.h"
@@ -70,6 +71,16 @@ struct s2n_security_policy {
      * https://www.rfc-editor.org/rfc/rfc8446#section-4.2.7
      */
     const struct s2n_ecc_preferences *ecc_preferences;
+    /* This field determines what public keys are allowed for use. It restricts
+     * both the type of the key (Elliptic Curve, RSA w/ Encryption, RSA PSS) and
+     * the size of the key. Note that this field structure is likely to change
+     * until https://github.com/aws/s2n-tls/issues/4435 is closed.
+     */
+    const struct s2n_certificate_key_preferences *certificate_key_preferences;
+    /* This field controls whether the certificate_signature_preferences apply 
+     * to local certs loaded on configs.
+     */
+    bool certificate_preferences_apply_locally;
     bool rules[S2N_SECURITY_RULES_COUNT];
 };
 
@@ -83,6 +94,12 @@ struct s2n_security_policy_selection {
 
 extern struct s2n_security_policy_selection security_policy_selection[];
 
+/* Defaults as of 05/24 */
+extern const struct s2n_security_policy security_policy_20240501;
+extern const struct s2n_security_policy security_policy_20240502;
+extern const struct s2n_security_policy security_policy_20240503;
+
+extern const struct s2n_security_policy security_policy_20241106;
 extern const struct s2n_security_policy security_policy_20140601;
 extern const struct s2n_security_policy security_policy_20141001;
 extern const struct s2n_security_policy security_policy_20150202;
@@ -103,8 +120,14 @@ extern const struct s2n_security_policy security_policy_20190214_gcm;
 extern const struct s2n_security_policy security_policy_20190801;
 extern const struct s2n_security_policy security_policy_20190802;
 extern const struct s2n_security_policy security_policy_20230317;
-extern const struct s2n_security_policy security_policy_default_tls13;
-extern const struct s2n_security_policy security_policy_default_fips;
+extern const struct s2n_security_policy security_policy_20240331;
+extern const struct s2n_security_policy security_policy_20240417;
+extern const struct s2n_security_policy security_policy_20240416;
+extern const struct s2n_security_policy security_policy_20240603;
+extern const struct s2n_security_policy security_policy_20240730;
+extern const struct s2n_security_policy security_policy_20241001;
+extern const struct s2n_security_policy security_policy_20241001_pq_mixed;
+
 extern const struct s2n_security_policy security_policy_rfc9151;
 extern const struct s2n_security_policy security_policy_test_all;
 
@@ -130,6 +153,8 @@ extern const struct s2n_security_policy security_policy_aws_crt_sdk_ssl_v3;
 extern const struct s2n_security_policy security_policy_aws_crt_sdk_tls_10;
 extern const struct s2n_security_policy security_policy_aws_crt_sdk_tls_11;
 extern const struct s2n_security_policy security_policy_aws_crt_sdk_tls_12;
+extern const struct s2n_security_policy security_policy_aws_crt_sdk_tls_12_06_23;
+extern const struct s2n_security_policy security_policy_aws_crt_sdk_tls_12_06_23_pq;
 extern const struct s2n_security_policy security_policy_aws_crt_sdk_tls_13;
 
 extern const struct s2n_security_policy security_policy_kms_pq_tls_1_0_2019_06;
@@ -158,6 +183,10 @@ extern const struct s2n_security_policy security_policy_pq_tls_1_2_2023_10_07;
 extern const struct s2n_security_policy security_policy_pq_tls_1_2_2023_10_08;
 extern const struct s2n_security_policy security_policy_pq_tls_1_2_2023_10_09;
 extern const struct s2n_security_policy security_policy_pq_tls_1_2_2023_10_10;
+extern const struct s2n_security_policy security_policy_pq_tls_1_2_2024_10_07;
+extern const struct s2n_security_policy security_policy_pq_tls_1_2_2024_10_08;
+extern const struct s2n_security_policy security_policy_pq_tls_1_2_2024_10_08_gcm;
+extern const struct s2n_security_policy security_policy_pq_tls_1_2_2024_10_09;
 
 extern const struct s2n_security_policy security_policy_cloudfront_upstream;
 extern const struct s2n_security_policy security_policy_cloudfront_upstream_tls10;
@@ -175,6 +204,7 @@ extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2021_
 extern const struct s2n_security_policy security_policy_kms_tls_1_0_2018_10;
 extern const struct s2n_security_policy security_policy_kms_tls_1_2_2023_06;
 extern const struct s2n_security_policy security_policy_kms_fips_tls_1_2_2018_10;
+extern const struct s2n_security_policy security_policy_kms_fips_tls_1_2_2024_10;
 
 extern const struct s2n_security_policy security_policy_20190120;
 extern const struct s2n_security_policy security_policy_20190121;
@@ -193,5 +223,12 @@ int s2n_validate_kem_preferences(const struct s2n_kem_preferences *kem_preferenc
 S2N_RESULT s2n_validate_certificate_signature_preferences(const struct s2n_signature_preferences *s2n_certificate_signature_preferences);
 S2N_RESULT s2n_security_policy_get_version(const struct s2n_security_policy *security_policy,
         const char **version);
+/* Checks to see if a certificate has a signature algorithm that's in our 
+ * certificate_signature_preferences list 
+ */
+S2N_RESULT s2n_security_policy_validate_certificate_chain(const struct s2n_security_policy *security_policy,
+        const struct s2n_cert_chain_and_key *cert_key_pair);
 S2N_RESULT s2n_security_policy_validate_cert_signature(
-        const struct s2n_security_policy *security_policy, const struct s2n_cert_info *info);
+        const struct s2n_security_policy *security_policy, const struct s2n_cert_info *info, s2n_error error);
+S2N_RESULT s2n_security_policy_validate_cert_key(
+        const struct s2n_security_policy *security_policy, const struct s2n_cert_info *info, s2n_error error);
